==> reprit-0.1.1-py3-none-any.whl: reprit/base.py <==
from collections import (OrderedDict,
                         abc)
from inspect import (_ParameterKind,
                     signature)
from typing import (Iterable,
                    Union)

from . import seekers
from .hints import (Constructor,
                    Domain,
                    FieldSeeker,
                    Initializer,
                    Map)


def generate_repr(constructor_or_initializer: Union[Constructor, Initializer],
                  *,
                  field_seeker: FieldSeeker = seekers.simple,
                  prefer_keyword: bool = False,
                  with_module_name: bool = False) -> Map[Domain, str]:
    """
    Generates `__repr__` method based on constructor/initializer parameters.

    We are assuming that no parameters data
    get thrown away during instance creation,
    so we can re-create it after.

    :param constructor_or_initializer: constructor/initializer method
    which parameters will be used in resulting representation.
    :param field_seeker: function that re-creates parameter value
    based on class instance and name.
    :param prefer_keyword: flag that specifies
    if positional-or-keyword parameters should be outputted
    as keyword ones when possible.
    :param with_module_name: flag that specifies
    if module name should be added.

    >>> from reprit.base import generate_repr
    >>> class Person:
    ...     def __init__(self, name, *, address=None):
    ...         self.name = name
    ...         self.address = address
    ...     __repr__ = generate_repr(__init__)
    >>> Person('Adrian')
    Person('Adrian', address=None)
    >>> Person('Mary', address='Somewhere on Earth')
    Person('Mary', address='Somewhere on Earth')
    >>> class ScoreBoard:
    ...     def __init__(self, first, *rest):
    ...         self.first = first
    ...         self.rest = rest
    ...     __repr__ = generate_repr(__init__,
    ...                              prefer_keyword=True)
    >>> ScoreBoard(1)
    ScoreBoard(first=1)
    >>> ScoreBoard(1, 40)
    ScoreBoard(1, 40)
    >>> class Student:
    ...     def __init__(self, name, group):
    ...         self.name = name
    ...         self.group = group
    ...     __repr__ = generate_repr(__init__,
    ...                              with_module_name=True)
    >>> Student('Kira', 132)
    reprit.base.Student('Kira', 132)
    >>> Student('Kira', 132)
    reprit.base.Student('Kira', 132)
    >>> Student('Naomi', 248)
    reprit.base.Student('Naomi', 248)
    >>> from reprit import seekers
    >>> class Account:
    ...     def __init__(self, id_, *, balance=0):
    ...         self.id = id_
    ...         self.balance = balance
    ...     __repr__ = generate_repr(__init__,
    ...                              field_seeker=seekers.complex_)
    >>> Account(1)
    Account(1, balance=0)
    >>> Account(100, balance=-10)
    Account(100, balance=-10)
    """
    if with_module_name:
        def to_class_name(cls: type) -> str:
            return cls.__module__ + '.' + cls.__qualname__
    else:
        def to_class_name(cls: type) -> str:
            return cls.__qualname__

    def __repr__(self: Domain) -> str:
        return (to_class_name(type(self))
                + '(' + ', '.join(to_arguments_strings(self)) + ')')

    parameters = OrderedDict(signature(constructor_or_initializer).parameters)
    # remove `cls`/`self`
    parameters.popitem(0)
    variadic_positional = next(
            (parameter
             for parameter in parameters.values()
             if parameter.kind is _ParameterKind.VAR_POSITIONAL),
            None)
    to_positional_argument_string = repr
    to_keyword_argument_string = '{}={!r}'.format

    def to_arguments_strings(object_: Domain) -> Iterable[str]:
        variadic_positional_unset = (
                variadic_positional is None
                or not field_seeker(object_, variadic_positional.name))
        for parameter_name, parameter in parameters.items():
            field = field_seeker(object_, parameter_name)
            if parameter.kind is _ParameterKind.POSITIONAL_ONLY:
                yield to_positional_argument_string(field)
            elif parameter.kind is _ParameterKind.POSITIONAL_OR_KEYWORD:
                if prefer_keyword and variadic_positional_unset:
                    yield to_keyword_argument_string(parameter_name, field)
                else:
                    yield to_positional_argument_string(field)
            elif parameter.kind is _ParameterKind.VAR_POSITIONAL:
                if isinstance(field, abc.Iterator):
                    # we don't want to exhaust iterator
                    yield '...'
                else:
                    yield from map(to_positional_argument_string, field)
            elif parameter.kind is _ParameterKind.KEYWORD_ONLY:
                yield to_keyword_argument_string(parameter_name, field)
            else:
                yield from map(to_keyword_argument_string,
                               field.keys(), field.values())

    return __repr__

==> reprit-0.3.1-py3-none-any.whl: reprit/base.py <==
from collections import (OrderedDict,
                         abc)
from inspect import (_ParameterKind,
                     signature)
from typing import (Iterable,
                    Union)

from . import seekers
from .hints import (Constructor,
                    Domain,
                    FieldSeeker,
                    Initializer,
                    Map)


def generate_repr(method: Union[Constructor, Initializer],
                  *,
                  field_seeker: FieldSeeker = seekers.simple,
                  prefer_keyword: bool = False,
                  with_module_name: bool = False) -> Map[Domain, str]:
    """
    Generates ``__repr__`` method based on constructor/initializer parameters.

    We are assuming that no parameters data
    get thrown away during instance creation,
    so we can re-create it after.

    :param method:
        constructor/initializer method
        which parameters will be used in resulting representation.
    :param field_seeker:
        function that re-creates parameter value
        based on class instance and name.
    :param prefer_keyword:
        flag that specifies
        if positional-or-keyword parameters should be outputted
        as keyword ones when possible.
    :param with_module_name:
        flag that specifies if module name should be added.

    >>> from reprit.base import generate_repr
    >>> class Person:
    ...     def __init__(self, name, *, address=None):
    ...         self.name = name
    ...         self.address = address
    ...     __repr__ = generate_repr(__init__)
    >>> Person('Adrian')
    Person('Adrian', address=None)
    >>> Person('Mary', address='Somewhere on Earth')
    Person('Mary', address='Somewhere on Earth')
    >>> class ScoreBoard:
    ...     def __init__(self, first, *rest):
    ...         self.first = first
    ...         self.rest = rest
    ...     __repr__ = generate_repr(__init__,
    ...                              prefer_keyword=True)
    >>> ScoreBoard(1)
    ScoreBoard(first=1)
    >>> ScoreBoard(1, 40)
    ScoreBoard(1, 40)
    >>> class Student:
    ...     def __init__(self, name, group):
    ...         self.name = name
    ...         self.group = group
    ...     __repr__ = generate_repr(__init__,
    ...                              with_module_name=True)
    >>> Student('Kira', 132)
    reprit.base.Student('Kira', 132)
    >>> Student('Naomi', 248)
    reprit.base.Student('Naomi', 248)
    >>> from reprit import seekers
    >>> class Account:
    ...     def __init__(self, id_, *, balance=0):
    ...         self.id = id_
    ...         self.balance = balance
    ...     __repr__ = generate_repr(__init__,
    ...                              field_seeker=seekers.complex_)
    >>> Account(1)
    Account(1, balance=0)
    >>> Account(100, balance=-10)
    Account(100, balance=-10)
    >>> import json
    >>> class Object:
    ...     def __init__(self, value):
    ...         self.value = value
    ...     @property
    ...     def serialized(self):
    ...         return json.dumps(self.value)
    ...     @classmethod
    ...     def from_serialized(cls, serialized):
    ...         return cls(json.loads(serialized))
    ...     __repr__ = generate_repr(from_serialized)
    >>> Object.from_serialized('0')
    Object.from_serialized('0')
    >>> Object.from_serialized('{"key": "value"}')
    Object.from_serialized('{"key": "value"}')
    """
    if with_module_name:
        def to_class_name(cls: type) -> str:
            return cls.__module__ + '.' + cls.__qualname__
    else:
        def to_class_name(cls: type) -> str:
            return cls.__qualname__

    unwrapped_method = (method.__func__
                        if isinstance(method, (classmethod, staticmethod))
                        else method)
    method_name = unwrapped_method.__name__
    parameters = OrderedDict(signature(unwrapped_method).parameters)

    if method_name in ('__init__', '__new__'):
        # remove `cls`/`self`
        parameters.popitem(0)

        def __repr__(self: Domain) -> str:
            return (to_class_name(type(self))
                    + '(' + ', '.join(to_arguments_strings(self)) + ')')
    else:
        if isinstance(method, classmethod):
            # remove `cls`
            parameters.popitem(0)

        def __repr__(self: Domain) -> str:
            return (to_class_name(type(self)) + '.' + method_name
                    + '(' + ', '.join(to_arguments_strings(self)) + ')')

    variadic_positional = next(
            (parameter
             for parameter in parameters.values()
             if parameter.kind is _ParameterKind.VAR_POSITIONAL),
            None)
    to_positional_argument_string = repr
    to_keyword_argument_string = '{}={!r}'.format

    def to_arguments_strings(object_: Domain) -> Iterable[str]:
        variadic_positional_unset = (
                variadic_positional is None
                or not field_seeker(object_, variadic_positional.name))
        for parameter_name, parameter in parameters.items():
            field = field_seeker(object_, parameter_name)
            if parameter.kind is _ParameterKind.POSITIONAL_ONLY:
                yield to_positional_argument_string(field)
            elif parameter.kind is _ParameterKind.POSITIONAL_OR_KEYWORD:
                if prefer_keyword and variadic_positional_unset:
                    yield to_keyword_argument_string(parameter_name, field)
                else:
                    yield to_positional_argument_string(field)
            elif parameter.kind is _ParameterKind.VAR_POSITIONAL:
                if isinstance(field, abc.Iterator):
                    # we don't want to exhaust iterator
                    yield '...'
                else:
                    yield from map(to_positional_argument_string,
                                   field() if callable(field) else field)
            elif parameter.kind is _ParameterKind.KEYWORD_ONLY:
                yield to_keyword_argument_string(parameter_name, field)
            else:
                yield from map(to_keyword_argument_string,
                               field.keys(), field.values())

    return __repr__
